# Copyright (C) 2019 Majormode.  All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from majormode.mercurius.constant.place import AddressComponentName
from majormode.mercurius.model.place import Place
from majormode.perseus.model.geolocation import GeoPoint
from majormode.perseus.model.locale import DEFAULT_LOCALE
from majormode.perseus.constant.http import HttpMethod
from majormode.perseus.service.base_http_handler import HttpRequest
from majormode.perseus.service.base_http_handler import HttpRequestHandler
from majormode.perseus.service.base_http_handler import http_request
from majormode.perseus.utils import cast

from majormode.mercurius.service.place.place_service import PlaceService


class PlaceServiceHttpRequestHandler(HttpRequestHandler):
    # @http_request(r'^/place$',
    #               http_method=HttpMethod.POST,
    #               authentication_required=True,
    #               signature_required=True)
    # def add_place(self, request):
    #     address_components = request.get_argument(
    #         'address',
    #         data_type=HttpRequest.ArgumentDataType.list,
    #         is_required=False)
    #
    #     if address_components:
    #         try:
    #             address_components = [
    #                 (cast.string_to_enum(property_name, AddressComponentName), property_value)
    #                 for property_name, property_value in address_components ]
    #         except:
    #             raise PlaceService.InvalidArgumentException('Invalid address information')
    #
    #     locale = request.get_argument('locale',
    #             data_type=HttpRequest.ArgumentDataType.locale,
    #             is_required=False,
    #             default_value=DEFAULT_LOCALE)
    #
    #     location_url_value = request.get_argument('ll',
    #             data_type=HttpRequest.ArgumentDataType.string,
    #             is_required=False)
    #
    #     if location_url_value:
    #         location_coordinates = location_url_value.split(',')
    #         if len(location_coordinates) != 2:
    #             raise PlaceService.InvalidArgumentException('The location MUST have the form "lon,lat"')
    #
    #         location = GeoPoint(float(location_coordinates[0]), float(location_coordinates[1]))
    #
    #     return PlaceService().add_place(request.app_id, request.account_id,
    #             address_components=address_components,
    #             locale=locale,
    #             location=location_url_value and location)


    @http_request(r'^/place$',
                  http_method=HttpMethod.GET,
                  authentication_required=False,
                  signature_required=True)
    def get_places(self, request):
        categories = request.get_argument(
            'categories',
            data_type=HttpRequest.ArgumentDataType.list,
            is_required=False)

        # intent = request.get_argument('intent',
        #         data_type=HttpRequest.ArgumentDataType.enumeration,
        #         enumeration=SearchIntent,
        #         is_required=False,
        #         default_value=SearchIntent.browse)

#        if intent in (SearchIntent.checkin, SearchIntent.social) \
#           and request.account_id is None:
#            raise PlaceService.InvalidArgumentException('The intent of the query is incompatible with anonymous user')

        latitude = request.get_argument(
            'latitude',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=False)

        longitude = request.get_argument(
            'longitude',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=False)

        location = (latitude and longitude) and GeoPoint(latitude, longitude)

        radius = request.get_argument(
            'radius',
            data_type=HttpRequest.ArgumentDataType.decimal,
            is_required=False,
            default_value=PlaceService.DEFAULT_SEARCH_RADIUS)

        bounds_url_value = request.get_argument(
            'bounds',
            data_type=HttpRequest.ArgumentDataType.string,
            is_required=False)

        if bounds_url_value:
            bounds_coordinates = bounds_url_value.split(',')
            if len(bounds_coordinates) != 4:
                raise PlaceService.InvalidArgumentException('The bounds MUST have the form "NE-lat,NE-lon,SW-lat,SW-lon"')

            bounds = (GeoPoint(float(bounds_coordinates[0]), float(bounds_coordinates[1])),
                      GeoPoint(float(bounds_coordinates[2]), float(bounds_coordinates[3])))
        else:
            bounds = None

        ip_address = request.get_argument(
            'ip_address',
            data_type=HttpRequest.ArgumentDataType.ipv4,
            is_required=False)

        limit = request.get_argument(
            'limit',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False,
            default_value=PlaceService.DEFAULT_LIMIT)

        offset = request.get_argument(
            'offset',
            data_type=HttpRequest.ArgumentDataType.integer,
            is_required=False,
            default_value=0)

        include_address = request.get_argument(
            'include_address',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=False,
            default_value=False)

        include_contacts = request.get_argument(
            'include_contacts',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=False,
            default_value=False)

        include_photos = request.get_argument(
            'include_photos',
            data_type=HttpRequest.ArgumentDataType.boolean,
            is_required=False,
            default_value=False)

        locale = request.get_argument(
            'locale',
            data_type=HttpRequest.ArgumentDataType.locale,
            is_required=False,
            default_value=DEFAULT_LOCALE)

        sync_time = request.get_argument(
            'sync_time',
            data_type=HttpRequest.ArgumentDataType.timestamp,
            is_required=False)

        team_id = request.get_argument(
            'team_id',
            data_type=HttpRequest.ArgumentDataType.uuid,
            is_required=False)

        return PlaceService().get_places(
            request.app_id,
            account_id=request.account_id,
            bounds=bounds,
            categories=categories,
            #intent=intent,
            location=location, radius=radius,
            limit=limit, offset=offset,
            include_address=include_address, include_contacts=include_contacts, include_photos=include_photos,
            locale=locale,
            ip_address=ip_address,
            sync_time=sync_time,
            team_id=team_id)


    # @http_request(r'^/place/sync$',
    #               http_method=HttpRequest.HttpMethod.PUT,
    #               authentication_required=True,
    #               signature_required=True)
    # def sync_place(self, request):
    #     place = Place.from_json(request.body)
    #
    #     optimistic_match = request.get_argument('optimistic_match',
    #             argument_passing=HttpRequest.ArgumentPassing.query_string,
    #             data_type=HttpRequest.ArgumentDataType.boolean,
    #             is_required=False,
    #             default_value=False)
    #
    #     team_id = request.get_argument('team_id',
    #             argument_passing=HttpRequest.ArgumentPassing.query_string,
    #             data_type=HttpRequest.ArgumentDataType.uuid,
    #             is_required=False)
    #
    #     return PlaceService().sync_place(request.app_id, request.account_id, place,
    #             optimistic_match=optimistic_match,
    #             team_id=team_id)
